@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright © 2002-2014 Sylvain Rochet
+ *  Copyright © 2002-2015 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,17 +15,17 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; see the file COPYING. If not, see
+ *  <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include <string.h>
 
-#include "config.h"
-
 #include "global.h"
 #include "callback.h"
 #include "client.h"
@@ -47,127 +47,86 @@ void Callback_ShowConfiguration(GtkWidget* menuitem)  {
 }
 
 
-/* **** Show all monopd servers */
-void Callback_ShowMetaServer(GtkWidget* button)  {
-	(void)button;
-
-	create_connection_metaserver(CONNECT_TYPE_META_GETLIST);
-}
-
-
 /* **** Get games from all monopd server */
-void Callback_RefreshGameMetaServer(GtkWidget* button)  {
+void Callback_ConnectMetaServer(GtkWidget* button)  {
 	(void)button;
 
-	create_connection_metaserver(CONNECT_TYPE_META_GETGAME);
-}
-
-
-/* **** Get games from custom server */
-void Callback_GetgamesButton(GtkWidget* button)  {
-
-	guint32 port;
-	gchar *host;
-
-	host = g_strstrip( g_strdup( gtk_entry_get_text( GTK_ENTRY( g_object_get_data(G_OBJECT(button), "host") ) ) ) );
-	port = atoi( gtk_entry_get_text( GTK_ENTRY( g_object_get_data(G_OBJECT(button), "port") ) ) );
-
-	create_connection_get_games(host, port);
-	g_free(host);
+	if (global->phase < PHASE_GAMECREATE) {
+		if (global->phase < PHASE_GETGAMES) {
+			interface_create_getgamespage(false);
+		}
+		create_connection_metaserver();
+	}
 }
 
 
 /* get games: event "row_select" */
 void Callback_GetGame_Select(GtkTreeSelection *selection, gpointer data)  {
-	GtkTreeIter iter;
 	GtkTreeModel *model;
-	gchar *host, *gametype;
-	gint32 port, gameid;
+	gint32 gameid, canbejoined, canbewatched;
 	(void)data;
 
-	if (!gtk_tree_selection_get_selected(selection, &model, &iter)) {
-		if(global->selected_host) g_free(global->selected_host);
-		global->selected_host = NULL;
-		global->selected_port = 0;
-		if(global->selected_gametype) g_free(global->selected_gametype);
-		global->selected_gametype = NULL;
-		global->selected_gameid = 0;
+	GtkWidget *button = g_object_get_data(G_OBJECT(global->MainVerticalBox), "connect_button");
+
+	if (!gtk_tree_selection_get_selected(selection, &model, &global->selected_game)) {
+		if (button) {
+			gtk_button_set_label(GTK_BUTTON(button), "Select a game");
+			gtk_widget_set_sensitive(button, FALSE);
+		}
 		return;
 	}
 
-	gtk_tree_model_get(model, &iter, GAMELIST_COLUMN_HOST, &host, GAMELIST_COLUMN_PORT, &port, GAMELIST_COLUMN_GAMETYPE, &gametype, GAMELIST_COLUMN_ID, &gameid, -1);
+	gtk_tree_model_get(model, &global->selected_game, GAMELIST_COLUMN_GAMEID, &gameid, GAMELIST_COLUMN_CANBEJOINED, &canbejoined, GAMELIST_COLUMN_CANBEWATCHED, &canbewatched, -1);
 
-	if(global->selected_host) g_free(global->selected_host);
-	global->selected_host = host;
-	global->selected_port = port;	
-	if(global->selected_gametype) g_free(global->selected_gametype);
-	global->selected_gametype = gametype;
-	global->selected_gameid = gameid;
+	if (button) {
+		if (gameid < 0) {
+			gtk_button_set_label(GTK_BUTTON(button), "Create game");
+			gtk_widget_set_sensitive(button, TRUE);
+		} else {
+			if (canbejoined) {
+				gtk_button_set_label(GTK_BUTTON(button), "Join game");
+				gtk_widget_set_sensitive(button, TRUE);
+			} else if (canbewatched) {
+				gtk_button_set_label(GTK_BUTTON(button), "Watch game");
+				gtk_widget_set_sensitive(button, TRUE);
+			} else {
+				gtk_button_set_label(GTK_BUTTON(button), "This game does not allow spectators");
+				gtk_widget_set_sensitive(button, FALSE);
+			}
+		}
+	}
 }
 
 
 /* **** Join/create a game */
 void Callback_ConnectButton(GtkWidget* button)  {
 	(void)button;
-	gint32 connectid;
-	gchar *text;
+	gchar *cmd;
+	gchar *host, *gametype;
+	gint32 port, gameid, canbejoined, canbewatched;
 
-	if (global->game_connectid) {
+	gtk_tree_model_get(GTK_TREE_MODEL(global->game_store), &global->selected_game, GAMELIST_COLUMN_HOST, &host, GAMELIST_COLUMN_PORT, &port, GAMELIST_COLUMN_GAMETYPE, &gametype, GAMELIST_COLUMN_GAMEID, &gameid,
+		   GAMELIST_COLUMN_CANBEJOINED, &canbejoined, GAMELIST_COLUMN_CANBEWATCHED, &canbewatched, -1);
+
+	if (! (gameid < 0 || canbejoined || canbewatched)) {
 		return;
 	}
 
-	if(!global->selected_host) return;
-	if(!global->selected_port) return;
-	if(!global->selected_gameid) return;
-	if(global->selected_gameid < 0 && !global->selected_gametype) return;
-
-	/* close custom server connection */
-	if (global->customserver_connectid) {
-		client_disconnect(global->customserver_connectid);
-		global->customserver_connectid = 0;
-	}
-	/* close meta server connection */
-	if (global->metaserver_connectid) {
-		client_disconnect(global->metaserver_connectid);
-		global->metaserver_connectid = 0;
-	}
-
-	if(! client_connect(&connectid, global->selected_host, global->selected_port) )   return;
-	connection[connectid]->type = CONNECT_TYPE_MONOPD_GAME;
-
-	/* set client protocol */
-	text = g_strdup_printf("client name=\"gtkAtlantic\" version=\"%s\" protocol=\"monopd\"\n", VERSION);
-	client_send(connectid, text, strlen(text));
-	g_free(text);
-
-	/* set nickname */
-	text = g_strdup_printf(".n%s\n", config->nickname);
-	client_send(connectid, text, strlen(text));
-	g_free(text);
-
-	global->game_connectid = connectid;
-	global->reconnected = FALSE;
-
 	/* create a game */
-	if(global->selected_gameid < 0)  {
-		text = g_strdup_printf(".gn%s\n", global->selected_gametype);
-		client_send(connectid, text, strlen(text));
-		g_free(text);
+	if (gameid < 0)  {
+		cmd = g_strdup_printf(".n%s\n.gn%s\n", config->nickname, gametype);
 	}
-
 	/* join a game */
-	if(global->selected_gameid > 0)  {
-		text = g_strdup_printf(".gj%d\n", global->selected_gameid);
-		client_send(connectid, text, strlen(text));
-		g_free(text);
+	else if (canbejoined) {
+		cmd = g_strdup_printf(".n%s\n.gj%d\n", config->nickname, gameid);
+	}
+	/* watch a game */
+	else {
+		cmd = g_strdup_printf(".n%s\n.gS%d\n", config->nickname, gameid);
 	}
 
-	g_free(global->selected_host);
-	g_free(global->selected_gametype);
-	global->selected_host = 0;
-	global->selected_port = 0;
-	global->selected_gametype = 0;
-	global->selected_gameid = 0;
+	global->reconnected = FALSE;
+	client_connect(CONNECT_TYPE_MONOPD_GAME, host, port, cmd);
 }
 
 
@@ -175,15 +134,7 @@ void Callback_ConnectButton(GtkWidget* button)  {
 void Callback_StartButton(GtkWidget* button)  {
 	(void)button;
 
-	client_send(global->game_connectid, ".gs\n", 4);
-}
-
-
-/* **** Enter in the real program */
-void Callback_EnterButton(GtkWidget* button)  {
-	(void)button;
-
-	interface_create_getgamespage();
+	client_send(global->game_connect, ".gs\n");
 }
 
 
@@ -201,7 +152,7 @@ void Callback_SendButton(GtkWidget* Button, GtkWidget *entry) {
 		tmp2 = g_strconcat(tmp, "\n", NULL);
 		g_free(tmp);
 		tmp = tmp2;
-		client_send(global->game_connectid, tmp, strlen(tmp) );
+		client_send(global->game_connect, tmp);
 	}
 	else
 		parse_specific_send_message(tmp);
@@ -211,81 +162,22 @@ void Callback_SendButton(GtkWidget* Button, GtkWidget *entry) {
 }
 
 
-/* *** Un/Float Server Message Box*/
-void CallBack_toggle_float_message(GtkWidget* widget) {
-	(void)widget;
-
-	if(global->phase != PHASE_GAMEPLAY) return;
-
-	currentgame->message_float = !currentgame->message_float;
-	interface_message_and_chat_float_unfloat();
-}
-
-
-/* *** Un/Float Chat Box*/
-void CallBack_toggle_float_chat(GtkWidget* widget) {
-	(void)widget;
-
-	if(global->phase != PHASE_GAMEPLAY) return;
-
-	currentgame->chat_float = !currentgame->chat_float;
-	interface_message_and_chat_float_unfloat();
-}
-
-
-/* *** "delete_event" signal window message */
-void CallBack_delete_window_message(GtkWidget* widget) {
-
-	gtk_widget_hide( GTK_WIDGET(global->MessageBox) );
-	gtk_widget_reparent(GTK_WIDGET(global->MessageBox), global->Trick);
-	gtk_widget_destroy(widget);
-	widget = 0;
-	currentgame->message_float = 0;
-	interface_message_and_chat_float_unfloat();
-}
-
-
-/* *** "delete_event" signal window chat */
-void CallBack_delete_window_chat(GtkWidget* widget) {
-
-	gtk_widget_hide( GTK_WIDGET(currentgame->ChatBox) );
-	gtk_widget_reparent(GTK_WIDGET(currentgame->ChatBox), global->Trick);
-	gtk_widget_destroy(widget);
-	widget = 0;
-	currentgame->chat_float = 0;
-	interface_message_and_chat_float_unfloat();
-}
-
-
-/* *** "destroy" signal window message */
-void CallBack_destroy_window_message(GtkWidget* widget) {
-	(void)widget;
-}
-
-
-/* *** "destroy" signal window chat */
-void CallBack_destroy_window_chat(GtkWidget* widget) {
-	(void)widget;
-}
-
-
 /* *** save and apply the config */
 void CallBack_ok_button_configwindow(GtkWidget* widget)  {
 	(void)widget;
 
 	gchar *filename, *errormsg;
 	FILE *file;
-	guint32 token_animate_speed;
 
-	token_animate_speed = (guint32) gtk_adjustment_get_value(GTK_ADJUSTMENT( g_object_get_data(G_OBJECT(global->ConfigWin), "config_gameinterface_tokenanimatespeed") ) );
-
-	filename = g_strconcat(global->path_home, "gtkatlantic.conf", NULL);
+	if (!global->path_home) {
+		return;
+	}
+	filename = g_strconcat(global->path_home, "/gtkatlantic.conf", NULL);
 
 	file = fopen(filename, "wt");
-	if(!file)  {
-
-		errormsg = g_strdup_printf("[ERROR] Can't write config file at \"%s\"", filename);
-		text_insert_message(errormsg, strlen(errormsg));
+	if (!file) {
+		errormsg = g_strdup_printf("Can't write config file at \"%s\"", filename);
+		interface_set_infolabel(errormsg, "b00000", false);
 		g_free(errormsg);
 		g_free(filename);
 		return;
@@ -294,20 +186,16 @@ void CallBack_ok_button_configwindow(GtkWidget* widget)  {
 
 	fprintf(file, "player nickname=\"%s\"\n",
 		gtk_entry_get_text( GTK_ENTRY( g_object_get_data(G_OBJECT(global->ConfigWin), "config_player_nickname") ) )  );
-	fprintf(file, "metaserver autogetgames=\"%d\" autogetlist=\"%d\" sendclientversion=\"%d\"\n",
-		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( g_object_get_data(G_OBJECT(global->ConfigWin), "config_metaserver_autogetgames") ) ),
-		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( g_object_get_data(G_OBJECT(global->ConfigWin), "config_metaserver_autogetlist") ) ),
+	fprintf(file, "metaserver autoconnect=\"%d\" sendclientversion=\"%d\"\n",
+		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( g_object_get_data(G_OBJECT(global->ConfigWin), "config_metaserver_autoconnect") ) ),
 		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( g_object_get_data(G_OBJECT(global->ConfigWin), "config_metaserver_sendclientversion") ) )  );
-	fprintf(file, "customserver host=\"%s\" port=\"%d\" autogetgames=\"%d\"\n",
+	fprintf(file, "customserver host=\"%s\" port=\"%d\" autoconnect=\"%d\"\n",
 		gtk_entry_get_text( GTK_ENTRY( g_object_get_data(G_OBJECT(global->ConfigWin), "config_customserver_host") ) ),
 		atoi( gtk_entry_get_text( GTK_ENTRY( g_object_get_data(G_OBJECT(global->ConfigWin), "config_customserver_port") ) ) ),
-		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( g_object_get_data(G_OBJECT(global->ConfigWin), "config_customserver_autogetgames") ) )  );
-	fprintf(file, "buffer message=\"%d\" chat=\"%d\"\n",
-		atoi( gtk_entry_get_text( GTK_ENTRY( g_object_get_data(G_OBJECT(global->ConfigWin), "config_buffersize_servermessages") ) ) ),
-		atoi( gtk_entry_get_text( GTK_ENTRY( g_object_get_data(G_OBJECT(global->ConfigWin), "config_buffersize_chat") ) ) ) );
+		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( g_object_get_data(G_OBJECT(global->ConfigWin), "config_customserver_autoconnect") ) )  );
 	fprintf(file, "interface playerlist=\"%s\" token_speed=\"%d\" token_transparency=\"%d\"\n",
 		gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT( g_object_get_data(G_OBJECT(global->ConfigWin), "config_gameinterface_playerlistposition") ) ),
-		token_animate_speed,
+		(guint32)gtk_adjustment_get_value(GTK_ADJUSTMENT( g_object_get_data(G_OBJECT(global->ConfigWin), "config_gameinterface_tokenanimatespeed") ) ),
 		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( g_object_get_data(G_OBJECT(global->ConfigWin), "config_gameinterface_tokentransparency") ) )  );
 
 	fclose(file);
@@ -330,47 +218,36 @@ void CallBack_configwindow_destroyed(GtkWidget* widget)  {
 void CallBack_button_command_pressed(GtkWidget* button) {
 
 	gchar *ptr = g_object_get_data(G_OBJECT(button), "command");
-	client_send(global->game_connectid, ptr, strlen(ptr) );
-
-	if(! strcmp(ptr, ".E\n") )
-		display_clear_buttons();
+	client_send(global->game_connect, ptr);
 }
 
 
 /* *** button command pressed */
 void Callback_AuctionButton_Absolute(GtkWidget* button, GtkWidget* entry)  {
-	(void)button;
-
-	gchar *tosend, *auction, *money;
-
-	auction = g_object_get_data(G_OBJECT(entry), "auctionid");
-
-	money = g_strdup( gtk_entry_get_text( GTK_ENTRY(entry) ) );
-	g_strstrip(money);
-
-	tosend = g_strconcat(".ab", auction, ":", money, "\n", NULL);
-
-	client_send(global->game_connectid, tosend, strlen(tosend) );
-
-	g_free(money);
-	g_free(tosend);
+	gchar *tosend;
+	gint32 auction, money;
 
+	auction = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button), "auctionid"));
+	money = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
 	gtk_entry_set_text(GTK_ENTRY(entry), "");
+
+	tosend = g_strdup_printf(".ab%d:%d\n", auction, money);
+	client_send(global->game_connect, tosend);
+	g_free(tosend);
 }
 
 
 /* *** button command pressed */
 void Callback_AuctionButton_Relative(GtkWidget* button)  {
+	gchar *tosend;
+	gint32 auction, money;
 
-	gchar *tosend, *auction;
-	gint32 money;
+	auction = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button), "auctionid"));
+	money = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(currentgame->BoardCenter), "auction_highbid"))
+		+ GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button), "value_add"));
 
-	money = GPOINTER_TO_INT( g_object_get_data(G_OBJECT(currentgame->BoardCenter), "auction_highbid") )
-		+ GPOINTER_TO_INT( g_object_get_data(G_OBJECT(button), "value_add") );
-	auction = g_object_get_data(G_OBJECT( g_object_get_data(G_OBJECT(button), "auction_entry") ), "auctionid");
-
-	tosend = g_strdup_printf(".ab%s:%d\n", auction, money);
-	client_send(global->game_connectid, tosend, strlen(tosend) );
+	tosend = g_strdup_printf(".ab%d:%d\n", auction, money);
+	client_send(global->game_connect, tosend);
 	g_free(tosend);
 }
 
@@ -410,7 +287,7 @@ void CallBack_PopupMenuEstate(GtkWidget* widget)  {
 	}
 
 	if (cmd) {
-		client_send(global->game_connectid, cmd, strlen(cmd));
+		client_send(global->game_connect, cmd);
 		g_free(cmd);
 	}
 }
@@ -455,27 +332,27 @@ void Callback_ClicOnPlayer(GtkWidget* widget, GdkEvent *event)  {
 
 /* *** callback clic on player */
 void CallBack_PopupMenuPlayer(GtkWidget* widget)  {
-	guint32 id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(gtk_widget_get_parent(widget)), "playerid"));
+	player *p = g_object_get_data(G_OBJECT(gtk_widget_get_parent(widget)), "player");
 	gint32 action = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "action"));
 	gchar *cmd = NULL;
 
 	switch (action) {
 		case PLAYER_ACTION_TRADE:
-			cmd = g_strdup_printf(".Tn%d\n", id);
+			cmd = g_strdup_printf(".Tn%d\n", p->playerid);
 			break;
 		case PLAYER_ACTION_VERSION:
-			cmd = g_strdup_printf("!version %s\n", global->player[ get_player_slot_with_playerid(id) ].name);
+			cmd = g_strdup_printf("!version %s\n", p->name);
 			break;
 		case PLAYER_ACTION_DATE:
-			cmd = g_strdup_printf("!date %s\n", global->player[ get_player_slot_with_playerid(id) ].name);
+			cmd = g_strdup_printf("!date %s\n", p->name);
 			break;
 		case PLAYER_ACTION_PING:
-			cmd = g_strdup_printf("!ping %s\n", global->player[ get_player_slot_with_playerid(id) ].name);
+			cmd = g_strdup_printf("!ping %s\n", p->name);
 			break;
 	}
 
 	if (cmd) {
-		client_send(global->game_connectid, cmd, strlen(cmd));
+		client_send(global->game_connect, cmd);
 		g_free(cmd);
 	}
 }
@@ -490,7 +367,7 @@ void Callback_toggle_boolean_gameoption(GtkWidget* widget)  {
 	id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "id"));
 	if (id > 0) {
 		text = g_strdup_printf(".gc%d:%d\n", id, gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget) )  );
-		client_send(global->game_connectid, text, strlen(text) );
+		client_send(global->game_connect, text);
 		g_free(text);
 		return;
 	}
@@ -499,12 +376,50 @@ void Callback_toggle_boolean_gameoption(GtkWidget* widget)  {
 	cmd = g_object_get_data(G_OBJECT(widget), "command");
 	if (cmd) {
 		text = g_strdup_printf("%s%d\n", cmd, gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget) )  );
-		client_send(global->game_connectid, text, strlen(text) );
+		client_send(global->game_connect, text);
 		g_free(text);
 	}
 }
 
 
+/* *** callback for the entry menu connect to server */
+void CallBack_menu_connect_to_server(GtkWidget* widget) {
+	(void)widget;
+
+	if (global->phase < PHASE_GAMECREATE)
+		interface_create_connectoserver();
+}
+
+
+/* *** callback for the connect to server menu entry */
+void CallBack_response_connect_to_server_window(GtkWidget* widget, gint response, GtkWidget *Win) {
+	(void)widget;
+
+	if (global->phase < PHASE_GAMECREATE && response == GTK_RESPONSE_OK) {
+		GtkWidget *host = g_object_get_data(G_OBJECT(Win), "host");
+		GtkWidget *port = g_object_get_data(G_OBJECT(Win), "port");
+
+		if (global->phase < PHASE_GETGAMES) {
+			interface_create_getgamespage(false);
+		}
+
+		create_connection_get_games((gchar*)gtk_entry_get_text(GTK_ENTRY(host)),
+			atoi(gtk_entry_get_text(GTK_ENTRY(port))));
+	}
+
+	gtk_widget_destroy(Win);
+}
+
+
+/* *** callback for the entry menu connect to server */
+void CallBack_menu_public_server_list(GtkWidget* widget) {
+	(void)widget;
+
+	if (global->phase == PHASE_GETGAMES)
+		interface_create_publicserverlist();
+}
+
+
 /* *** callback for the entry menu change nickname */
 void CallBack_menu_change_nickname(GtkWidget* widget)  {
 	(void)widget;
@@ -525,10 +440,9 @@ void CallBack_response_changenick_window(GtkWidget* widget, gint response, GtkWi
 
 		nickname = g_strdup( gtk_entry_get_text(GTK_ENTRY( Entry ) )  );
 
-		if(strcmp(global->player[ get_player_slot_with_playerid(global->my_playerid) ].name, nickname) )
-		{
+		if (strcmp(player_from_id(global->my_playerid)->name, nickname)) {
 			sendstr = g_strdup_printf(".n%s\n", nickname);
-			client_send(global->game_connectid, sendstr, strlen(sendstr) );
+			client_send(global->game_connect, sendstr);
 			g_free(sendstr);
 		}
 
@@ -542,69 +456,55 @@ void CallBack_response_changenick_window(GtkWidget* widget, gint response, GtkWi
 /* *** callback for the entry menu reconnect */
 void CallBack_menu_reconnect(GtkWidget* widget)  {
 	(void)widget;
-	gchar *host, *tmp, *cookie, *filename;
+	gchar buf[64];
+	gchar *filename, *host, *cmd;
 	guint32 port;
 	FILE *file;
-	gint32 connectid;
-	gchar *text;
 
-	if (global->game_connectid) {
+	if (global->phase > PHASE_GETGAMES) {
 		return;
 	}
 
-	if(global->phase > PHASE_GETGAMES) return;
+	if (!global->path_home) {
+		return;
+	}
 
 	/* read cookie file */
-	filename = g_strconcat(global->path_home, "cookie", NULL);
+	filename = g_strconcat(global->path_home, "/cookie", NULL);
 	file = fopen(filename, "rt");
 	g_free(filename);
 
-	if(!file)  return;
-
-	host = g_malloc0(4096);
-	fscanf(file, "%s\n", host);
-
-	tmp = g_malloc0(4096);
-	fscanf(file, "%s\n", tmp);
-	port = atoi(tmp);
-	g_free(tmp);
-
-	cookie = g_malloc0(4096);
-	fscanf(file, "%s\n", cookie);
-
-	fclose(file);
-
-	/* close custom server connection */
-	if (global->customserver_connectid) {
-		client_disconnect(global->customserver_connectid);
-		global->customserver_connectid = 0;
-	}
-	/* close meta server connection */
-	if (global->metaserver_connectid) {
-		client_disconnect(global->metaserver_connectid);
-		global->metaserver_connectid = 0;
-	}
-
-	/* rejoin game */
-	if(! client_connect(&connectid, host, port) )   {
-
-		g_free(host);
-		g_free(cookie);
+	if (!file) {
 		return;
 	}
 
-	g_free(host);
-	connection[connectid]->type = CONNECT_TYPE_MONOPD_GAME;
-	global->game_connectid = connectid;
+	fscanf(file, "%s\n", buf);
+	host = g_strdup(buf);
+
+	fscanf(file, "%s\n", buf);
+	port = atoi(buf);
+
+	fscanf(file, "%s\n", buf);
+	cmd = g_strdup_printf(".R%s\n", buf);
+
+	fclose(file);
+
+	/* rejoin game */
 	global->reconnected = TRUE;
-
-	/* send cookie */
-	text = g_strdup_printf(".R%s\n", cookie);
-	client_send(connectid, text, strlen(text));
-	g_free(text);
+	client_connect(CONNECT_TYPE_MONOPD_GAME, host, port, cmd);
+	g_free(host);
+}
 
 
-	g_free(cookie);
+void CallBack_menu_AddPlayer(GtkWidget *widget) {
+	player *p = g_object_get_data(G_OBJECT(widget), "player");
+	if (!p) {
+		return;
+	}
+
+	gchar *cmd = g_strdup_printf(".gu%d\n", p->playerid);
+	client_send(global->game_connect, cmd);
+	g_free(cmd);
 }
 
 
@@ -619,13 +519,13 @@ void CallBack_trade_button(GtkWidget *button)  {
 	switch(command) {
 		case TRADE_ACTION_REJECT:
 			sendstr = g_strdup_printf(".Tr%d\n", tradeid);
-			client_send(global->game_connectid, sendstr, strlen(sendstr));
+			client_send(global->game_connect, sendstr);
 			g_free(sendstr);
 			break;
 			
 		case TRADE_ACTION_ACCEPT:
 			sendstr = g_strdup_printf(".Ta%d:%d\n", tradeid, currentgame->trade[tradeslot].revision);
-			client_send(global->game_connectid, sendstr, strlen(sendstr));
+			client_send(global->game_connect, sendstr);
 			g_free(sendstr);
 			break;
 
@@ -640,7 +540,7 @@ void CallBack_trade_button(GtkWidget *button)  {
 				sendstr = g_strdup_printf(".Tc%d:%d:%d\n", tradeid, currentgame->trade[tradeslot].select_aux, currentgame->trade[tradeslot].select_from);
 
 			if(sendstr)  {
-				client_send(global->game_connectid, sendstr, strlen(sendstr));
+				client_send(global->game_connect, sendstr);
 				g_free(sendstr);
 			}
 			break;
@@ -674,7 +574,8 @@ void CallBack_trade_sub_component(GtkWidget *combo)  {
 void CallBack_trade_update_component(GtkWidget *button)  {
 
 	gchar *sendstr;
-	guint32 tradeid, estateid, cardid, target, from, to, tradeslot, money;
+	guint32 tradeid, estateid, cardid, tradeslot, money;
+	player *target, *from, *to;
 
 	tradeslot = get_trade_slot_with_tradeid( GPOINTER_TO_INT( g_object_get_data( G_OBJECT(button), "tradeid" )  )  );
 	tradeid = currentgame->trade[tradeslot].tradeid;
@@ -684,41 +585,42 @@ void CallBack_trade_update_component(GtkWidget *button)  {
 		case TRADE_TYPE_ESTATE:
 
 			estateid = get_estateid_by_estatename( (gchar*)gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT(g_object_get_data( G_OBJECT(button), "name_estate" ) ))  );
-			target = get_playerid_by_playername( (gchar*)gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT(g_object_get_data( G_OBJECT(button), "name_target_player" ) ))  );
+			target = player_from_name((gchar*)gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(g_object_get_data(G_OBJECT(button), "name_target_player"))));
 
-			sendstr = g_strdup_printf(".Te%d:%d:%d\n", tradeid, estateid, target);
-			client_send(global->game_connectid, sendstr, strlen(sendstr));
+			sendstr = g_strdup_printf(".Te%d:%d:%d\n", tradeid, estateid, target->playerid);
+			client_send(global->game_connect, sendstr);
 			g_free(sendstr);
 			break;
 
 		case TRADE_TYPE_MONEY:
 
-			from = get_playerid_by_playername( (gchar*)gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT(g_object_get_data( G_OBJECT(button), "name_from_player" ) ))  );
-			to = get_playerid_by_playername( (gchar*) gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT(g_object_get_data( G_OBJECT(button), "name_to_player" ) ))  );
+			from = player_from_name((gchar*)gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(g_object_get_data(G_OBJECT(button), "name_from_player"))));
+			to = player_from_name((gchar*)gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(g_object_get_data(G_OBJECT(button), "name_to_player"))));
 			money = atoi( gtk_entry_get_text( g_object_get_data( G_OBJECT(button), "amount_money" ) )  );
 
-			sendstr = g_strdup_printf(".Tm%d:%d:%d:%d\n", tradeid, from, to, money);
-			client_send(global->game_connectid, sendstr, strlen(sendstr));
+			sendstr = g_strdup_printf(".Tm%d:%d:%d:%d\n", tradeid, from->playerid, to->playerid, money);
+			client_send(global->game_connect, sendstr);
 			g_free(sendstr);
 			break;
 
 		case TRADE_TYPE_CARD:
 
 			cardid = atoi( gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT(g_object_get_data( G_OBJECT(button), "name_card" ) ))  );
-			target = get_playerid_by_playername( (gchar*) gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT(g_object_get_data( G_OBJECT(button), "name_target_player" ) )) );
-			sendstr = g_strdup_printf(".Tc%d:%d:%d\n", tradeid, cardid, target);
-			client_send(global->game_connectid, sendstr, strlen(sendstr));
+			target = player_from_name((gchar*)gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(g_object_get_data(G_OBJECT(button), "name_target_player"))));
+			sendstr = g_strdup_printf(".Tc%d:%d:%d\n", tradeid, cardid, target->playerid);
+			client_send(global->game_connect, sendstr);
 			g_free(sendstr);
 			break;
 	}
 }
 
 
-void CallBack_logo_page_destroyed(GtkWidget *widget)  {
+void CallBack_get_games_page_destroyed(GtkWidget *widget)  {
 	(void)widget;
 
-	eng_frame_destroy(global->logo_frame);
-	intro_free_pngs();
+	g_object_unref(global->server_store);
+	global->game_store = NULL;
+	global->server_store = NULL;
 }
 
 
@@ -726,17 +628,11 @@ void CallBack_logo_page_destroyed(GtkWidget *widget)  {
 void CallBack_menu_LeaveGame(GtkWidget* widget)  {
 	(void)widget;
 
-	gchar *sendstr;
-
-	if(global->phase < PHASE_GAMECREATE)  return;
-
-	if(global->phase >= PHASE_GAMEPLAY)  {
-
-		sendstr = g_strdup_printf(".gx\n");
-		client_send(global->game_connectid, sendstr, strlen(sendstr));
-		g_free(sendstr);
+	if (global->phase < PHASE_GAMECREATE) {
+		return;
 	}
 
+	client_send(global->game_connect, ".gx\n");
 	game_quit();
 }
 
@@ -751,25 +647,13 @@ void CallBack_estates_tree(GtkWidget* widget) {
 }
 
 
-/* *** callback for the entry menu declare bankrupcy */
-void CallBack_menu_DeclareBankrupcy(GtkWidget* widget)  {
-	(void)widget;
-
-	gchar *sendstr;
-
-	if(global->phase < PHASE_GAMEPLAY) return;
-
-	sendstr = g_strdup_printf(".D\n");
-	client_send(global->game_connectid, sendstr, strlen(sendstr));
-	g_free(sendstr);
-}
 
 
 void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)  {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	gint32 id = -1;
-	GtkWidget *parent, *info, *Alignment, *Label;
+	GtkWidget *parent, *info, *Label;
 	gchar *Text;
 
 	(void)data;
@@ -798,10 +682,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 		gtk_box_pack_start(GTK_BOX(parent), info, TRUE, TRUE, 0);
 
 		/* default text */
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new("Select an estate...");
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		gtk_widget_show_all(info);
 		return;
@@ -810,10 +694,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 	/* name */
 	Text = g_strdup_printf("%s", currentgame->estate[id].name);
 
-	Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-	gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 	Label = gtk_label_new(Text);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_START);
+	gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 	g_free(Text);
 
@@ -822,10 +706,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 	if(currentgame->estate[id].group >= 0)  Text = g_strdup_printf("%s\n", currentgame->group[ currentgame->estate[id].group ].name);
 	else  Text = g_strdup("\n");
 
-	Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-	gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 	Label = gtk_label_new(Text);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_START);
+	gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 	g_free(Text);
 
@@ -837,10 +721,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 		else  if(currentgame->estate[id].owner > 0)  Text = g_strdup("State: owned");
 		else  Text = g_strdup("State: unknow");
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -849,12 +733,12 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 	if(currentgame->estate[id].price)  {
 
 		if(currentgame->estate[id].owner <= 0)  Text = g_strdup("Owner: no");
-		else  Text = g_strdup_printf("Owner: %s", global->player[ get_player_slot_with_playerid( currentgame->estate[id].owner ) ].name);
+		else  Text = g_strdup_printf("Owner: %s", player_from_id(currentgame->estate[id].owner)->name);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -864,10 +748,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 
 		Text = g_strdup_printf("\nPrice: %d", currentgame->estate[id].price);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -877,10 +761,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 
 		Text = g_strdup_printf("Money: %d", currentgame->estate[id].money);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -890,60 +774,60 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 
 		Text = g_strdup_printf("\nRent with no house: %d", currentgame->estate[id].rent[0]);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 
 
 		Text = g_strdup_printf("Rent with 1 house: %d", currentgame->estate[id].rent[1]);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 
 
 		Text = g_strdup_printf("Rent with 2 houses: %d", currentgame->estate[id].rent[2]);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 
 
 		Text = g_strdup_printf("Rent with 3 houses: %d", currentgame->estate[id].rent[3]);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 
 
 		Text = g_strdup_printf("Rent with 4 houses: %d", currentgame->estate[id].rent[4]);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 
 
 		Text = g_strdup_printf("Rent with a hotel: %d", currentgame->estate[id].rent[5]);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -953,10 +837,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 
 		Text = g_strdup_printf("\nBuy house price: %d", currentgame->estate[id].houseprice);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -965,10 +849,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 
 		Text = g_strdup_printf("Sell house price: %d", currentgame->estate[id].sellhouseprice);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -978,10 +862,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 
 		Text = g_strdup_printf("Number of houses: %d", currentgame->estate[id].houses);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -991,10 +875,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 
 		Text = g_strdup_printf("\nMortgage price: %d", currentgame->estate[id].mortgageprice);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
@@ -1003,10 +887,10 @@ void CallBack_EstateTree_SelectRow(GtkTreeSelection *selection, gpointer data)
 
 		Text = g_strdup_printf("Unmortgage price: %d", currentgame->estate[id].unmortgageprice);
 
-		Alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_box_pack_start(GTK_BOX(info), Alignment, FALSE, TRUE, 0);
 		Label = gtk_label_new(Text);
-		gtk_container_add(GTK_CONTAINER(Alignment), Label);
+		gtk_widget_set_halign(Label, GTK_ALIGN_START);
+		gtk_widget_set_valign(Label, GTK_ALIGN_START);
+		gtk_box_pack_start(GTK_BOX(info), Label, FALSE, TRUE, 0);
 
 		g_free(Text);
 	}
