@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright © 2002-2014 Sylvain Rochet
+ *  Copyright © 2002-2015 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,12 +15,19 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; see the file COPYING. If not, see
+ *  <http://www.gnu.org/licenses/>.
  */
 
+#ifndef CLIENT_H
+#define CLIENT_H
+
+#include <global.h>
+
 /* ** prototypes for connect/disconnect to server */
-gboolean client_connect(gint32 *idconnect, gchar* host, gint32 port);
-gboolean client_disconnect(gint32 id);
-gboolean client_event(GIOChannel *channel, GIOCondition condition, gpointer data);
-gboolean client_send(gint32 id, gchar *data, gint32 lenght_data);
+void client_connect(guint8 type, gchar* host, gint32 port, gchar *cmd);
+void client_disconnect(connection *c);
+gboolean client_event(GIOChannel *channel, GIOCondition condition, connection *c);
+void client_send(connection *c, gchar *data);
+
+#endif /* CLIENT_H */
\ No newline at end of file
