@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright © 2002-2014 Sylvain Rochet
+ *  Copyright © 2002-2015 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,10 +15,13 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; see the file COPYING. If not, see
+ *  <http://www.gnu.org/licenses/>.
  */
 
+#ifndef DISPLAY_H
+#define DISPLAY_H
+
 #include <gtk/gtk.h>
 
 void display_init();
@@ -30,3 +33,5 @@ void display_estate(gint32 estateid);
 void display_title(gchar *text);
 void display_text(gchar *message);
 void display_add_button(gchar *caption, gchar *command, gboolean enabled);
+
+#endif /* DISPLAY_H */
\ No newline at end of file
