@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright © 2002-2014 Sylvain Rochet
+ *  Copyright © 2002-2015 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,10 +15,12 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; see the file COPYING. If not, see
+ *  <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -30,18 +32,11 @@
 
 #include <libxml/parser.h>
 
-#include "config.h"
-
 #include "load.h"
 #include "global.h"
 #include "readpng.h"
 #include "parser.h"
 
-#ifdef WIN32
-#undef PACKAGE_DATA_DIR
-#define PACKAGE_DATA_DIR "./data"
-#endif
-
 /* parse a config file */
 gint8 parse_file_config(gchar *filename)  {
 
@@ -62,63 +57,67 @@ gint8 parse_file_config(gchar *filename)  {
 		if(! strcmp(ident, "player") )  {
 
 			tmp = parser_get_data(str, "nickname");
-			if(config->nickname) g_free(config->nickname);
-			config->nickname = tmp;
+			if (tmp) {
+				if(config->nickname) g_free(config->nickname);
+				config->nickname = tmp;
+			}
 
 		} else if(! strcmp(ident, "metaserver") )  {
 
-			tmp = parser_get_data(str, "autogetgames");
-			config->metaserver_autogetgames = atoi( (gchar*)tmp );
-			g_free(tmp);
-
-			tmp = parser_get_data(str, "autogetlist");
-			config->metaserver_autogetlist = atoi( (gchar*)tmp );
-			g_free(tmp);
+			tmp = parser_get_data(str, "autoconnect");
+			if (tmp) {
+				config->metaserver_autoconnect = atoi( (gchar*)tmp );
+				g_free(tmp);
+			}
 
 			tmp = parser_get_data(str, "sendclientversion");
-			config->metaserver_sendclientversion = atoi( (gchar*)tmp );
-			g_free(tmp);
+			if (tmp) {
+				config->metaserver_sendclientversion = atoi( (gchar*)tmp );
+				g_free(tmp);
+			}
 
 		} else if(! strcmp(ident, "customserver") )  {
 
 			tmp = parser_get_data(str, "host");
-			if(config->getgames_host) g_free(config->getgames_host);
-			config->getgames_host = tmp;
+			if (tmp) {
+				if(config->getgames_host) g_free(config->getgames_host);
+				config->getgames_host = tmp;
+			}
 
 			tmp = parser_get_data(str, "port");
-			config->getgames_port = atoi( (gchar*)tmp );
-			g_free(tmp);
+			if (tmp) {
+				config->getgames_port = atoi( (gchar*)tmp );
+				g_free(tmp);
+			}
 
-			tmp = parser_get_data(str, "autogetgames");
-			config->getgames_autogetgames = atoi( (gchar*)tmp );
-			g_free(tmp);
-
-		} else if(! strcmp(ident, "buffer") )  {
-
-			tmp = parser_get_data(str, "message");
-			config->message_max_lines = atoi( (gchar*)tmp );
-			g_free(tmp);
-
-			tmp = parser_get_data(str, "chat");
-			config->chat_max_lines = atoi( (gchar*)tmp );
-			g_free(tmp);
+			tmp = parser_get_data(str, "autoconnect");
+			if (tmp) {
+				config->getgames_autoconnect = atoi( (gchar*)tmp );
+				g_free(tmp);
+			}
 
 		} else if(! strcmp(ident, "interface") )  {
 
 			tmp = parser_get_data(str, "playerlist");
-			if(! strcmp( tmp, "left") )
-				config->game_playerlist_position = GAME_PLAYERLIST_POS_LEFT;
-			if(! strcmp( tmp, "right") )
-				config->game_playerlist_position = GAME_PLAYERLIST_POS_RIGHT;
-			g_free(tmp);
+			if (tmp) {
+				if(! strcmp( tmp, "left") )
+					config->game_playerlist_position = GAME_PLAYERLIST_POS_LEFT;
+				if(! strcmp( tmp, "right") )
+					config->game_playerlist_position = GAME_PLAYERLIST_POS_RIGHT;
+				g_free(tmp);
+			}
 
 			tmp = parser_get_data(str, "token_speed");
-			config->game_token_animation_speed = atoi( (gchar*)tmp );
-			g_free(tmp);
+			if (tmp) {
+				config->game_token_animation_speed = atoi( (gchar*)tmp );
+				g_free(tmp);
+			}
 
 			tmp = parser_get_data(str, "token_transparency");
-			config->game_token_transparency = atoi( (gchar*)tmp );
-			g_free(tmp);
+			if (tmp) {
+				config->game_token_transparency = atoi( (gchar*)tmp );
+				g_free(tmp);
+			}
 		}
 
 		g_free(ident);
@@ -136,17 +135,13 @@ gint8 xmlparse_file_interface()  {
 
 	xmlDocPtr doc;
 	xmlNodePtr cur, cur2;
-	guint8 ident = 0xff;
+	guint8 ident;
 	guint32 color;
 	gint32 xoffset, yoffset;
-	gchar *filename;
 	xmlChar *tmp;
 
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/interface.xml", NULL);
-	doc = xmlParseFile(filename);
-	g_free(filename);
-
-	if(doc == NULL)  return -1;
+	doc = xmlParseFile(PACKAGE_DATA_DIR "/interface.xml");
+	if (doc == NULL) return -1;
 
 	cur = xmlDocGetRootElement(doc);
 	if (cur == NULL) {
@@ -165,25 +160,6 @@ gint8 xmlparse_file_interface()  {
 		/* ===== PNGFILES ===== */
 		/* ==================== */
 
-		/* parse pngfile_logo */
-		if(! xmlStrcmp(cur->name, (xmlChar*)"pngfile_logo") )  {
-
-			data->pngfile_logo_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
-
-			cur2 = cur->xmlChildrenNode;
-			do {
-
-				if(! xmlStrcmp(cur2->name, (xmlChar*)"buffer") )  {
-
-					tmp = xmlGetProp(cur2, (xmlChar*)"id");
-					if( xmlStrcmp(tmp, (xmlChar*)"full")  )  g_free(data->pngfile_logo_filename);
-					g_free(tmp);
-				}
-
-				cur2 = cur2 -> next;
-			} while(cur2 != NULL);
-		}
-
 		/* parse pngfile_board */
 		if(! xmlStrcmp(cur->name, (xmlChar*)"pngfile_board") )  {
 
@@ -232,6 +208,9 @@ gint8 xmlparse_file_interface()  {
 					tmp = xmlGetProp(cur2, (xmlChar*)"id");
 					ident = atoi( (gchar*)tmp );
 					g_free(tmp);
+					if (ident > data->token_max) {
+						data->token_max = ident;
+					}
 
 					tmp = xmlGetProp(cur2, (xmlChar*)"x");
 					data->pngfile_token_x[ident]  = atoi( (gchar*)tmp );
@@ -699,42 +678,6 @@ gint8 xmlparse_file_interface()  {
 }
 
 
-/* read all intro pngs images */
-gint8 intro_load_pngs()   {
-	_png_imagedata png_imagedata;
-	gchar *filename;
-	gint8 ret = 0;
-
-	memset(&png_imagedata, 0, sizeof(_png_imagedata));
-
-	/* ---- load logo file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_logo_filename, NULL);
-	if (readpng_read(filename, &png_imagedata)) {
-		ret = -1;
-		goto free_and_return;
-	}
-
-	data->logo_width = readpng_get_width(&png_imagedata);
-	data->logo_height = readpng_get_height(&png_imagedata);
-	data->pngfile_logo_buf = readpng_get_rgba(&png_imagedata);
-
-free_and_return:
-	g_free(filename);
-	readpng_cleanup(&png_imagedata);
-	return ret;
-}
-
-
-/* free all intro pngs images */
-void intro_free_pngs()  {
-
-	if (data->pngfile_logo_buf) {
-		g_free(data->pngfile_logo_buf);
-		data->pngfile_logo_buf = NULL;
-	}
-}
-
-
 /* read all game pngs images */
 gint8 game_load_pngs()   {
 	_png_imagedata png_imagedata;
@@ -747,7 +690,7 @@ gint8 game_load_pngs()   {
 	}
 
 	/* ---- load board file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_board_filename, NULL);
+	filename = g_strconcat(PACKAGE_DATA_DIR "/", data->pngfile_board_filename, NULL);
 	if (readpng_read(filename, &png_imagedata)) {
 		ret = -1;
 		goto free_and_return;
@@ -762,7 +705,7 @@ gint8 game_load_pngs()   {
 
 
 	/* ---- load token file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_token_filename, NULL);
+	filename = g_strconcat(PACKAGE_DATA_DIR "/", data->pngfile_token_filename, NULL);
 	if (readpng_read(filename, &png_imagedata)) {
 		ret = -1;
 		goto free_and_return;
@@ -777,7 +720,7 @@ gint8 game_load_pngs()   {
 
 
 	/* ---- load star file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_star_filename, NULL);
+	filename = g_strconcat(PACKAGE_DATA_DIR "/", data->pngfile_star_filename, NULL);
 	if (readpng_read(filename, &png_imagedata)) {
 		ret = -1;
 		goto free_and_return;
@@ -792,7 +735,7 @@ gint8 game_load_pngs()   {
 
 
 	/* ---- load star_m file (mortgaged) */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_star_m_filename, NULL);
+	filename = g_strconcat(PACKAGE_DATA_DIR "/", data->pngfile_star_m_filename, NULL);
 	if (readpng_read(filename, &png_imagedata)) {
 		ret = -1;
 		goto free_and_return;
@@ -807,7 +750,7 @@ gint8 game_load_pngs()   {
 
 
 	/* ---- load cards file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_card_filename, NULL);
+	filename = g_strconcat(PACKAGE_DATA_DIR "/", data->pngfile_card_filename, NULL);
 	if (readpng_read(filename, &png_imagedata)) {
 		ret = -1;
 		goto free_and_return;
@@ -823,7 +766,7 @@ gint8 game_load_pngs()   {
 
 
 	/* ---- load command file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_command_filename, NULL);
+	filename = g_strconcat(PACKAGE_DATA_DIR "/", data->pngfile_command_filename, NULL);
 	if (readpng_read(filename, &png_imagedata)) {
 		ret = -1;
 		goto free_and_return;
@@ -838,7 +781,7 @@ gint8 game_load_pngs()   {
 
 
 	/* ---- load horiz_houses file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_horiz_house_filename, NULL);
+	filename = g_strconcat(PACKAGE_DATA_DIR "/", data->pngfile_horiz_house_filename, NULL);
 	if (readpng_read(filename, &png_imagedata)) {
 		ret = -1;
 		goto free_and_return;
@@ -853,7 +796,7 @@ gint8 game_load_pngs()   {
 
 
 	/* ---- load vert_houses file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_vert_house_filename, NULL);
+	filename = g_strconcat(PACKAGE_DATA_DIR "/", data->pngfile_vert_house_filename, NULL);
 	if (readpng_read(filename, &png_imagedata)) {
 		ret = -1;
 		goto free_and_return;
