@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright © 2002-2014 Sylvain Rochet
+ *  Copyright © 2002-2015 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,10 +15,13 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; see the file COPYING. If not, see
+ *  <http://www.gnu.org/licenses/>.
  */
 
+#ifndef LOAD_H
+#define LOAD_H
+
 #include <gtk/gtk.h>
 
 #define DATAFILE_XMLROOTELEMENT (xmlChar*)"gtkatlantic"
@@ -26,7 +29,7 @@
 
 gint8 parse_file_config(gchar *filename);
 gint8 xmlparse_file_interface();
-gint8 intro_load_pngs();
-void intro_free_pngs();
 gint8 game_load_pngs();
 void game_free_pngs();
+
+#endif /* LOAD_H */
\ No newline at end of file
