@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright © 2002-2014 Sylvain Rochet
+ *  Copyright © 2002-2015 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,10 +15,12 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; see the file COPYING. If not, see
+ *  <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,8 +38,6 @@
 #include "callback.h"
 #include "readpng.h"
 
-#include "config.h"
-
 
 /* refresh themes database */
 void theme_build_database()   {
@@ -55,31 +55,31 @@ void theme_build_database()   {
 	}
 
 	/* user home theme */
-	path = g_strconcat(global->path_home, "themes/", NULL);
-	d = opendir (path);
+	if (global->path_home) {
+		path = g_strconcat(global->path_home, "/themes", NULL);
+		d = opendir (path);
 
-	if(d)  {
+		if(d)  {
 
-		while (1) {
+			while (1) {
 
-			dir = readdir(d);
-			if(!dir) break;
+				dir = readdir(d);
+				if(!dir) break;
 
-			if(dir->d_name[0] == '.')  continue;
+				if(dir->d_name[0] == '.')  continue;
 
-			path2 = g_strconcat(path, dir->d_name, "/", NULL);
-			theme_add_entry(path2);
-			g_free(path2);
+				path2 = g_strconcat(path, "/", dir->d_name, "/", NULL);
+				theme_add_entry(path2);
+				g_free(path2);
+			}
+			closedir (d);
 		}
-		closedir (d);
+		g_free(path);
 	}
-	g_free(path);
 
 	/* global theme */
-	path = g_strconcat(PACKAGE_DATA_DIR, "/themes/", NULL);
-	d = opendir (path);
-
-	if(d)  {
+	d = opendir (PACKAGE_DATA_DIR "/themes");
+	if (d) {
 
 		while (1) {
 
@@ -88,13 +88,12 @@ void theme_build_database()   {
 
 			if(dir->d_name[0] == '.')  continue;
 
-			path2 = g_strconcat(path, dir->d_name, "/", NULL);
+			path2 = g_strconcat(PACKAGE_DATA_DIR "/themes/", dir->d_name, "/", NULL);
 			theme_add_entry(path2);
 			g_free(path2);
 		}
 		closedir (d);
 	}
-	g_free(path);
 }
 
 
@@ -229,8 +228,7 @@ void theme_build_selection_win()  {
 						GtkWidget *Button;
 				GtkWidget *VBoxInfo;
 					GtkWidget *GridInfo;
-						GtkWidget *Alignment;
-							GtkWidget *Label;
+						GtkWidget *Label;
 				GtkWidget *PreviewFrame;
 
 	GtkWidget *tmp;
@@ -244,7 +242,6 @@ void theme_build_selection_win()  {
 
 	global->ThemeWin = ThemeWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(ThemeWin), "GtkAtlantic: Theme Selection");
-	gtk_window_set_icon(GTK_WINDOW(global->MainWin), global->icon_pixbuf);
 	g_signal_connect(G_OBJECT(ThemeWin), "delete_event", G_CALLBACK(CallBack_ThemeWin_Delete), NULL);
 	g_signal_connect(G_OBJECT(ThemeWin), "destroy", G_CALLBACK(CallBack_ThemeWin_Destroyed), NULL);
 
@@ -302,84 +299,83 @@ void theme_build_selection_win()  {
 	gtk_grid_set_row_spacing(GTK_GRID(GridInfo), 2);
 	gtk_box_pack_start(GTK_BOX(VBoxInfo), GridInfo, FALSE, FALSE, 0);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 0, 1, 1);
 	Label = gtk_label_new("Name:");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 0, 0, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 0, 1, 1);
 	Label = gtk_label_new("");
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
 	g_object_set_data(G_OBJECT(ThemeWin), "theme_name", Label);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 1, 0, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 1, 1, 1);
 	Label = gtk_label_new("Type:");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 0, 1, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 1, 1, 1);
 	Label = gtk_label_new("");
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
 	g_object_set_data(G_OBJECT(ThemeWin), "theme_type", Label);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 1, 1, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 2, 1, 1);
 	Label = gtk_label_new("Version:");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 0, 2, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 2, 1, 1);
 	Label = gtk_label_new("");
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
 	g_object_set_data(G_OBJECT(ThemeWin), "theme_version", Label);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 1, 2, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 3, 1, 1);
 	Label = gtk_label_new("Author:");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 0, 3, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 3, 1, 1);
 	Label = gtk_label_new("");
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
 	g_object_set_data(G_OBJECT(ThemeWin), "theme_author", Label);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 1, 3, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 4, 1, 1);
 	Label = gtk_label_new("Email:");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 0, 4, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 4, 1, 1);
 	Label = gtk_label_new("");
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
 	g_object_set_data(G_OBJECT(ThemeWin), "theme_email", Label);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 1, 4, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 5, 1, 1);
 	Label = gtk_label_new("Location:");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 0, 5, 1, 1);
 
-	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 5, 1, 1);
 	Label = gtk_label_new("");
+	gtk_widget_set_halign(Label, GTK_ALIGN_START);
+	gtk_widget_set_valign(Label, GTK_ALIGN_CENTER);
 	g_object_set_data(G_OBJECT(ThemeWin), "theme_location", Label);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_grid_attach(GTK_GRID(GridInfo), Label, 1, 5, 1, 1);
 
 	/* preview */
 	PreviewFrame = gtk_frame_new("Preview");
 	gtk_box_pack_start(GTK_BOX(VBoxInfo), PreviewFrame, TRUE, TRUE, 0);
-
-	Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-	g_object_set_data(G_OBJECT(ThemeWin), "theme_preview_container", Alignment);
-	gtk_widget_set_size_request(Alignment, 400, 245);
-	gtk_container_add(GTK_CONTAINER(PreviewFrame), Alignment);
+	gtk_widget_set_size_request(PreviewFrame, 400, 245);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_preview_container", PreviewFrame);
 
 	tmp = gtk_label_new("No preview available");
+	gtk_widget_set_halign(tmp, GTK_ALIGN_CENTER);
+	gtk_widget_set_valign(tmp, GTK_ALIGN_CENTER);
 	g_object_set_data(G_OBJECT(ThemeWin), "theme_preview_data", tmp);
-	gtk_container_add(GTK_CONTAINER(Alignment), tmp);
+	gtk_container_add(GTK_CONTAINER(PreviewFrame), tmp);
 
 	g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_image", NULL);
 	str = g_strdup_printf("%d", -1);
@@ -437,7 +433,7 @@ void theme_preview_free()  {
 void theme_display(gint32 themeid)  {
 
 	GtkWidget *Label;
-	GtkWidget *Alignment;
+	GtkWidget *Frame;
 	GtkWidget *Preview;
 	guint8 *buf = 0;
 	gchar *tmp;
@@ -469,15 +465,15 @@ void theme_display(gint32 themeid)  {
 	/* free all preview data */
 	theme_preview_free();
 
-	Alignment = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_preview_container");
+	Frame = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_preview_container");
 
 	/* put new preview data */
 	if(! (buf = theme_preview_read(themeid, &width, &height)) )  {
 
 		Preview = gtk_label_new("No preview available");
 		g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_data", Preview);
-		gtk_container_add(GTK_CONTAINER(Alignment), Preview);
-		gtk_widget_show_all(Alignment);
+		gtk_container_add(GTK_CONTAINER(Frame), Preview);
+		gtk_widget_show_all(Frame);
 		return;
 	}
 
@@ -486,10 +482,12 @@ void theme_display(gint32 themeid)  {
 
 	Preview = gtk_drawing_area_new();
 	gtk_widget_set_size_request( GTK_WIDGET(Preview), width, height);
+	gtk_widget_set_halign(Preview, GTK_ALIGN_CENTER);
+	gtk_widget_set_valign(Preview, GTK_ALIGN_CENTER);
 	g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_data", Preview);
-	gtk_container_add(GTK_CONTAINER(Alignment), Preview);
+	gtk_container_add(GTK_CONTAINER(Frame), Preview);
 	g_signal_connect(G_OBJECT (Preview), "draw", G_CALLBACK(on_theme_preview_draw), NULL);
-	gtk_widget_show_all(Alignment);
+	gtk_widget_show_all(Frame);
 }
 
 
